//------------------------------------------------------------------
#property copyright "Copyright 2016, mladen - MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
//------------------------------------------------------------------
#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1  C'214,248,214'
#property indicator_color2  C'255,236,213'
#property indicator_color3  clrDarkSlateGray
#property indicator_color4  clrGreen
#property indicator_color5  clrRed
#property indicator_width1  10
#property indicator_width2  10
#property indicator_style3  STYLE_DASH
#property indicator_width4  3
#property indicator_width5  3
#property strict

enum enCalcType
{
   st_atr, // Use atr
   st_std, // Use standard deviation 
   st_ste, // Use standard error
   st_sam, // Custom strandard deviation - with sample correction
   st_nos  // Custom strandard deviation - without sample correction
   
};

extern int        period          = 10;              // Super trend period
extern double     multiplier      = 2.0;             // Super trend multiplier
extern int        midPricePeriod  = 21;              // Mid price period (1 for original super trend)
extern enCalcType Type            = st_sam;          // Calculate using?
extern bool       UseFill         = true;            // UseFill



double Up[],Dn[],mprice[],upFill[],dnFill[],trendu[],trendd[],Direction[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0, upFill); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1, dnFill); SetIndexStyle(1,DRAW_HISTOGRAM); 
   SetIndexBuffer(2, mprice);
   SetIndexBuffer(3, trendu);
   SetIndexBuffer(4, trendd);
   SetIndexBuffer(5, Up);
   SetIndexBuffer(6, Dn);     
   SetIndexBuffer(7, Direction); 
   IndicatorShortName("SuperTrend");
return(0);
}
int deinit() {  return(0);}

//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1); 

   //
   //
   //
   //
   //

   for(i=limit; i >= 0; i--)
   {
      double val=0;
         switch (Type)
         {
            case st_atr : val = iATR(NULL,0,period,i);                           break;
            case st_std : val = iStdDev(NULL,0,period,0,MODE_SMA,PRICE_CLOSE,i); break;
            case st_ste : val = iStdError(Close[i],period,i);                    break;
            default :     val = iDeviation(Close[i],period,Type==st_sam,i);
         }            
      double cprice    = Close[i];
             mprice[i] = (High[ArrayMaximum(High,midPricePeriod,i)]+Low[ArrayMinimum(Low,midPricePeriod,i)])*0.5;
             Up[i]     = mprice[i]+multiplier*val;
             Dn[i]     = mprice[i]-multiplier*val;
             
             //
             //
             //
             //
             //
         
             Direction[i] = (i<Bars-1) ? (cprice > Up[i+1]) ? 1 : (cprice < Dn[i+1]) ? -1 : Direction[i+1] : 0;
             if (Direction[i] ==  1) 
             { 
                Dn[i]     = fmax(Dn[i],Dn[i+1]); 
                Up[i]     = fmax(Up[i],Up[i+1]);
                trendu[i] = Up[i];
                trendd[i] = Dn[i];
             }
             if (Direction[i] == -1) 
             { 
                 Up[i]     = fmin(Up[i],Up[i+1]); 
                 Dn[i]     = fmin(Dn[i],Dn[i+1]); 
                 trendd[i] = Up[i];
                 trendu[i] = Dn[i];
                 
             }
             if (UseFill) { upFill[i] = trendu[i]; dnFill[i] = trendd[i];}
             
   }
return(0);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
// 
//
//
//
//

#define _devInstances 1
double workDev[][_devInstances];
double iDeviation(double value, int length, bool isSample, int i, int instanceNo=0)
{
   if (ArrayRange(workDev,0)!=Bars) ArrayResize(workDev,Bars); i=Bars-i-1; workDev[i][instanceNo] = value;
                 
   //
   //
   //
   //
   //
   
      double oldMean   = value;
      double newMean   = value;
      double squares   = 0; int k;
      for (k=1; k<length && (i-k)>=0; k++)
      {
         newMean  = (workDev[i-k][instanceNo]-oldMean)/(k+1)+oldMean;
         squares += (workDev[i-k][instanceNo]-oldMean)*(workDev[i-k][instanceNo]-newMean);
         oldMean  = newMean;
      }
      return(MathSqrt(squares/fmax(k-isSample,1)));
}

//
//
//
//
//

double workErr[][_devInstances];
double iStdError(double value, int length,int i, int instanceNo=0)
{
   if (ArrayRange(workErr,0)!=Bars) ArrayResize(workErr,Bars); i = Bars-i-1; workErr[i][instanceNo] = value;
                        
      //
      //
      //
      //
      //
                              
      double avgY     = workErr[i][instanceNo]; int j; for (j=1; j<length && (i-j)>=0; j++) avgY += workErr[i-j][instanceNo]; avgY /= j;
      double avgX     = length * (length-1) * 0.5 / length;
      double sumDxSqr = 0.00;
      double sumDySqr = 0.00;
      double sumDxDy  = 0.00;
   
      for (int k=0; k<length && (i-k)>=0; k++)
      {
         double dx = k-avgX;
         double dy = workErr[i-k][instanceNo]-avgY;
            sumDxSqr += (dx*dx);
            sumDySqr += (dy*dy);
            sumDxDy  += (dx*dy);
      }
      double err2 = (sumDySqr-(sumDxDy*sumDxDy)/sumDxSqr)/(length-2); 
      
   //
   //
   //
   //
   //
         
   if (err2 > 0)
         return(MathSqrt(err2));
   else  return(0.00);       
}

